#!/usr/bin/env python

"""
Convert data in UniFrac's sample file format to EBD sample file format.

Usage:
$ ./convertUniFracToEBD.py <input UniFrac file> <output EBD file>
"""

import sys
import fileinput

if len(sys.argv) != 3:
	print "  Usage: convertUniFracToEBD.py <unifrac file> <ebd file>\n"
	sys.exit()
	
inputFile = sys.argv[1]
outputFile = sys.argv[2]

# read UniFrac sample file
dict = {}
seqs = set([])
for line in fileinput.input([inputFile]):
	if line.strip() == '':
		continue
		
	lineSplit = line.split('\t')
	seqId = lineSplit[0]
	sampleId = lineSplit[1].strip()
	if len(lineSplit) == 3:
		weight = lineSplit[2].strip()
	else:
		weight = '1'
		
	if sampleId not in dict:
		dict[sampleId] = {}
		
	dict[sampleId][seqId] = weight
	seqs.add(seqId)

# write EBD sample file
fout = open(outputFile, 'w')
seqs = sorted(list(seqs))
for seq in seqs:
	fout.write('\t' + seq)
fout.write('\n')

for sample in sorted(dict.keys()):
	fout.write(sample)
	sampleData = dict[sample]
	for seq in seqs:
		if seq in sampleData:
			fout.write('\t' + sampleData[seq])
		else:
			fout.write('\t' + '0')
	fout.write('\n')
fout.close()

